@@ -11,7 +11,10 @@ __all__ = (
     'EUI48', 'EUI60', 'EUI64',
     'parse',
 )
-__version__ = '1.1.2'
+__version__ = '2.0.0'
+
+
+from functools import total_ordering as _total_ordering
 
 
 _HEX_DIGITS = "0123456789ABCDEFabcdef"
@@ -43,6 +46,7 @@ def _value_error(value, error, *classes):
     return ValueError(repr(value) + ' ' + error + ' ' + class_names)
 
 
+@_total_ordering
 class HWAddress:
     """Base class for hardware addresses.
 
@@ -61,19 +65,21 @@ class HWAddress:
             "xx:xx:xx:xx:xx:xx", "xxxx.xxxx.xxxx", and "xxxxxxxxxxxx".
     """
 
-    __slots__ = ('_address',)
+    __slots__ = ('_address', '__weakref__')
+
+    formats = ()
 
     def __init__(self, address):
         """Initialize the hardware address object with the address given.
 
         Arguments:
-            address: An ``int``, ``bytes``, or ``str`` representation
-                of the address, or another instance of the same class
-                of address. If a string, the ``formats`` attribute of
-                the class is used to parse it. If a byte string, it
-                is read in big-endian. If an integer, its value bytes
-                in big-endian are used as the address bytes. If an
-                instance of the same address class, its value is used.
+            address: An ``int``, ``bytes``, or ``str`` representation of
+                the address, or another instance of an address which is
+                either the same class, a subclass, or a superclass. If a
+                string, the ``formats`` attribute of the class is used
+                to parse it. If a byte string, it is read in big-endian.
+                If an integer, its value bytes in big-endian are used as
+                the address bytes.
 
         Raises:
             TypeError: If ``address`` is not one of the valid types.
@@ -83,7 +89,7 @@ class HWAddress:
                 integer with a value that is negative or too big.
         """
         if isinstance(address, int):
-            overflow = 1 << self.size
+            overflow = 1 << type(self).size
             if address >= overflow:
                 raise _value_error(address, 'is too big for', type(self))
             if address < 0:
@@ -91,28 +97,40 @@ class HWAddress:
             self._address = address
         elif isinstance(address, bytes):
             length = len(address)
-            size_in_bytes = (self.size + 7) >> 3
+            size_in_bytes = (type(self).size + 7) >> 3
             if length != size_in_bytes:
                 raise _value_error(address, 'has wrong length for', type(self))
-            offset = (8 - self.size) & 7
+            offset = (8 - type(self).size) & 7
             self._address = int.from_bytes(address, 'big') >> offset
-        elif isinstance(address, str):
+        elif isinstance(address, str) and len(type(self).formats):
             self._address, _ = _parse(address, type(self))
+        # Subclass being "cast" to superclass:
         elif isinstance(address, type(self)):
             self._address = address._address
+        # Superclass being "cast" to subclass:
+        elif (isinstance(address, HWAddress)
+        and   isinstance(self, type(address))):
+            self._address = address._address
         else:
             raise _type_error(address, type(self))
 
     def __repr__(self):
         """Represent the hardware address as an unambiguous string."""
-        return _name(self) + '(' + repr(str(self)) + ')'
+        try:
+            address = str(self)
+        except TypeError:
+            address = bytes(self)
+        return _name(self) + '(' + repr(address) + ')'
 
     def __str__(self):
         """Get the canonical human-readable string of this hardware address."""
+        formats = type(self).formats
+        if not len(formats):
+            raise TypeError(_name(self) + ' has no string format')
         result = []
-        offset = (4 - self.size) & 3
+        offset = (4 - type(self).size) & 3
         unconsumed_address_value = self._address << offset
-        for character in reversed(self.formats[0]):
+        for character in reversed(formats[0]):
             if character == 'x':
                 nibble = unconsumed_address_value & 0xf
                 result.append(_HEX_DIGITS[nibble])
@@ -123,8 +141,8 @@ class HWAddress:
 
     def __bytes__(self):
         """Get the big-endian byte string of this hardware address."""
-        offset = (8 - self.size) & 7
-        size_in_bytes = (self.size + 7) >> 3
+        offset = (8 - type(self).size) & 7
+        size_in_bytes = (type(self).size + 7) >> 3
         return (self._address << offset).to_bytes(size_in_bytes, 'big')
 
     def __int__(self):
@@ -134,19 +152,12 @@ class HWAddress:
     def __eq__(self, other):
         """Check if this hardware address is equal to another.
 
-        They are equal if they are instances of the same class
-        (including one being a subclass of the other), if they
-        have the same size, and if their addresses are equal.
+        Hardware addresses are equal if they are instances of the
+        same class, and their their raw bit strings are the same.
         """
-        if not isinstance(other, type(self)) or self.size != other.size:
-            return False
-        return self._address == other._address
-
-    def __ne__(self, other):
-        """Check if this hardware address is not equal to another."""
-        if not isinstance(other, type(self)) or self.size != other.size:
-            return True
-        return self._address != other._address
+        if not isinstance(other, HWAddress):
+            return NotImplemented
+        return type(self) == type(other) and self._address == other._address
 
     def __lt__(self, other):
         """Check if this hardware address is before another.
@@ -163,38 +174,21 @@ class HWAddress:
         """
         if not isinstance(other, HWAddress):
             return NotImplemented
-        this, that = _aligned_address_integers(self, other)
-        return this < that or (this == that and self.size < other.size)
+        class1 = type(self)
+        class2 = type(other)
+        size1 = class1.size
+        size2 = class2.size
+        bits1 = int(self)
+        bits2 = int(other)
+        if size1 > size2:
+            bits2 <<= size1 - size2
+        else:
+            bits1 <<= size2 - size1
+        return (bits1, size1, id(class1)) < (bits2, size2, id(class2))
 
-    def __le__(self, other):
-        """Check if this hardware address is before or equal to another."""
-        if not isinstance(other, HWAddress):
-            return NotImplemented
-        this, that = _aligned_address_integers(self, other)
-        return this < that or (this == that and self.size <= other.size)
-
-    def __gt__(self, other):
-        """Check if this hardware address is after another."""
-        if not isinstance(other, HWAddress):
-            return NotImplemented
-        this, that = _aligned_address_integers(self, other)
-        return this > that or (this == that and self.size > other.size)
-
-    def __ge__(self, other):
-        """Check if this hardware address is after or equal to another."""
-        if not isinstance(other, HWAddress):
-            return NotImplemented
-        this, that = _aligned_address_integers(self, other)
-        return this > that or (this == that and self.size >= other.size)
-
-
-def _aligned_address_integers(address1, address2):
-    size1 = address1.size
-    size2 = address2.size
-    if size1 > size2:
-        return (int(address1), int(address2) << (size1 - size2))
-    else:
-        return (int(address1) << (size2 - size1), int(address2))
+    def __hash__(self):
+        """Get the hash of this hardware address."""
+        return hash((type(self), self._address))
 
 
 class OUI(HWAddress):
@@ -217,7 +211,7 @@ class _StartsWithOUI(HWAddress):
     @property
     def oui(self):
         """Get the OUI part of this hardware address."""
-        return OUI(int(self) >> (self.size - OUI.size))
+        return OUI(int(self) >> (type(self).size - OUI.size))
 
 
 class CDI32(_StartsWithOUI):
@@ -267,19 +261,7 @@ class EUI48(_StartsWithOUI):
     )
 
 
-class MAC(EUI48):
-    """MAC address. A subclass of EUI48.
-
-    There is nothing wrong with using EUI48 for MAC addresses,
-    this is just provided as a convenience for the many users
-    who will look for "MAC address" without knowing about EUI.
-
-    But it is a subclass instead of just an alias because it
-    might be nice in some situations to distinguish in code
-    between MAC addresses and other uses of EUI-48.
-    """
-
-    __slots__ = ()
+MAC = EUI48
 
 
 class EUI60(_StartsWithOUI):
@@ -314,17 +296,17 @@ class EUI64(_StartsWithOUI):
 def parse(value, *classes):
     """Try parsing a value as several hardware address classes at once.
 
-    This lets you can just write
+    This lets you just write
 
-        address = hwaddress.parse(user_input, EUI64, EUI48, ...)
+        address = macaddress.parse(user_input, EUI64, EUI48, ...)
 
     instead of all of this:
 
         try:
-            address = hwaddress.EUI64(user_input)
+            address = macaddress.EUI64(user_input)
         except ValueError:
             try:
-                address = hwaddress.EUI48(user_input)
+                address = macaddress.EUI48(user_input)
             except ValueError:
                 ...
 
@@ -369,9 +351,9 @@ def _parse(string, *classes):
         raise ValueError('hardware address cannot be an empty string')
     candidates = {}
     for cls in classes:
-        for format in cls.formats:
-            if len(format) == length:
-                candidates.setdefault(format, cls)
+        for format_ in cls.formats:
+            if len(format_) == length:
+                candidates.setdefault(format_, cls)
     candidates = sorted(candidates.items())
     address = 0
     start = 0
